#ifndef DIRSCANNER_H
#define DIRSCANNER_H

#include <exec/types.h>
#include <dos/dosextens.h>

/* ====================================================================

   This is a collection of utilities which assist with controlling a
   FileRequestor.  (Its sort of a file-requestor without the GUI.)

   A DirScanner is a data-structure which can be used to scan directories.
   It creates a list of all files in the directory.

   The files are stored as Node structures (as defined in <exec/nodes.h>)
   where the simple filename is stored in Node.ln_Name, and the type
   of the file is stored in Node.ln_Type
      = FALSE -> Plain file
      = TRUE  -> Directory

   Users should first call 'DirScanner_Init() to initialize the data
   structure.

   DirScanner_SetPath() sets the directory path to read.
   Any subsequent calls to DirScanner_SetPath() will erase the
   current contents of the list and start over.

   DirScanner_Scan() reads one entry from the directory table (one file).
   If the file passes the filter test, it is added to the tail of the
   list.  Call this repeatedly to fill the structure.  (It doesn't
   do it all-at-once so you can update the file requestor as it fills.)
   If you want a sorted list, sort after each call to DirScanner_Scan().

   When all files have been read from the directory, DirScanner.mode
   is set to DS_MODE_WAITING.

   Finally, you should ALWAYS call 'DirScanner_CleanUp()' to release
   the allocated memory.

   Lee Willis, April 1992

   ========================================================*/

#include <exec/types.h>
#include <exec/lists.h>
#include <exec/nodes.h>


/* ============ DS_FullPathName ============================

   This is a handy little utility to return the fullpathname
   associated with a file-lock.

   'lock' is a file Lock on the file whose fullname is to
   be returned.

   'fib' is an allocated FileInfoBlock structure which can
   be used by this routine (i.e. its contents will be trashed)

   'buffer' is a character string which will contain the fullpathname.
   it needs to be large enough to handle the path. (Make it big!)

   'buffer' is returned.

*/
char *DS_FullPathName( struct FileLock          *CurrentDirLock,
                       struct FileInfoBlock *fib,
                       char                 *buffer );

/* ============ DS_PathFromLock ============================

   This is an even handier little utility to return the fullpathname
   associated with a file-lock.

   'lock' is a file Lock on the file whose fullname is to
   be returned.

   It allocates a string, and returns it.  You need to free the
   string with >>> Afree() <<<, not free().

*/

char* DS_PathFromLock( struct FileLock *lock );


/*============  DS_Filter =====================================

   This is a pointer to a file-filter function.
   The function should take a FileInfoBlock, and return TRUE
   if that file should be made part of the list.  This can
   be used to eliminate unwanted files from the list.
*/
typedef BOOL (*DS_Filter)( struct FileInfoBlock * );


/*=============== struct DirScanner =========================

   This is the main data structure.

   'filter' is a pointer to a DS_Filter function, or NULL to
   include all files.

   'fib' is a FileInfoBlock.  Treat it as private.

   'pathlock' is a Lock on the directory being scanned.  Treat it
   as read-only.

   'list' is the list of files of this directory which match the
   filter pattern.  Treat it as read-only.

   'mode' is the mode of the scan (treat it as read-only):

      = DS_MODE_READING : indicates that there are potentially more
                          files to read.
      = DS_MODE_WAITING : indicates there are no more files in this
                          directory.
*/

struct DirScanner
{
   struct MinList        list;      /* READ ONLY! list of files in dir. */
   char                 *pathname;  /* READ ONLY!         */
   struct FileLock      *pathlock;  /* READ ONLY!         */
   struct FileInfoBlock *fib;       /* PRIVATE!           */
   DS_Filter             filter;    /* User can set this. */
   short                 mode;      /* READ ONLY!         */
};

#define DS_MODE_READING 1 /* Directory scanner is reading a dir. */
#define DS_MODE_WAITING 0 /* Directory scanner is doing nothing. */


BOOL DirScanner_Init( struct DirScanner *ds,
                      DS_Filter          filter );

/* ========= DirScanner_SetPath ======================

   This sets the path of the scan. Pass in the pathname of the
   directory to scan, or

      DS_PARENTPATH : to go to the parent of the current directory.

      DS_DEVICEPATH : to list the devices available.

*/
BOOL DirScanner_SetPath( struct DirScanner *ds, char *path );

#define DS_PARENTPATH ((char*)-1)
#define DS_DEVICEPATH ((char*)-2)



BOOL DirScanner_SetChildPath( struct DirScanner *ds,
                              struct Node       *node );


struct Node* DirScanner_Scan( struct DirScanner *ds );
/*
   Returns the node of the file that was just scanned, or
   NULL at the end of a dir.
*/

void DirScanner_CleanUp( struct DirScanner *ds );

BOOL DSFilter_NoIcons( struct FileInfoBlock *fib );

#endif

